<?php
/**
 * Search Logger - ACP Module
 *
 * @package   SearchLogger
 * @copyright (c) 2025 salvocortesiano
 * @license   GNU General Public License, version 2 (GPL-2.0)
 */

namespace salvocortesiano\searchlogger\acp;

class main_module
{
    /** @var \phpbb\config\config */
    protected $config;

    /** @var \phpbb\db\driver\driver_interface */
    protected $db;

    /** @var \phpbb\user */
    protected $user;

    /** @var \phpbb\language\language */
    protected $language;

	protected $phpbb_container;

    /** @var \phpbb\template\template */
    protected $template;

    /** @var \phpbb\request\request */
    protected $request;

    /** @var string Custom form action */
    public $u_action;

    /** @var string phpBB tables prefix */
    protected $table_prefix;

    /** @var string tpl_name */
    public $tpl_name;

    /** @var string page_title */
    public $page_title;

    /**
     * Main ACP module
     *
     * @param int    $id   The module ID
     * @param string $mode The module mode
     */
    public function main($id, $mode)
    {
        global $config, $db, $user, $template, $request, $phpbb_container, $table_prefix;

        $this->config = $config;
        $this->db = $db;
        $this->user = $user;
        $this->language = $phpbb_container->get('language');
        $this->template = $template;
        $this->request = $request;
        $this->table_prefix = $table_prefix;

        $this->user->add_lang_ext('salvocortesiano/searchlogger', 'searchlogger');
        $this->language->add_lang('info_acp_searchlogger', 'salvocortesiano/searchlogger');

		$this->phpbb_container = $phpbb_container;

        // Set up the page
        $this->tpl_name = 'acp_searchlogger_' . $mode;
        $this->page_title = $this->language->lang('ACP_SEARCHLOGGER_TITLE');

        // Load the appropriate mode
        $this->{"mode_$mode"}();
    }

    /**
     * Settings mode
     */
    protected function mode_settings()
    {
        add_form_key('acp_searchlogger');
        
        if ($this->request->is_set_post('submit'))
        {
            if (!check_form_key('acp_searchlogger'))
            {
                trigger_error($this->language->lang('FORM_INVALID') . adm_back_link($this->u_action), E_USER_WARNING);
            }

            $this->config->set('searchlogger_enabled', $this->request->variable('searchlogger_enabled', 0));
            $this->config->set('searchlogger_retention_days', $this->request->variable('searchlogger_retention_days', 30));

            trigger_error($this->language->lang('ACP_SEARCHLOGGER_SETTINGS_SAVED') . adm_back_link($this->u_action));
        }

        $this->template->assign_vars(array(
            'U_ACTION'                    => $this->u_action,
            'SEARCHLOGGER_ENABLED'        => $this->config['searchlogger_enabled'],
            'SEARCHLOGGER_RETENTION_DAYS' => $this->config['searchlogger_retention_days'],
        ));
    }

    /**
     * View mode
     */
    protected function mode_view()
    {
        // Define how many logs to display per page
        $per_page = 25;
        $start = $this->request->variable('start', 0);

        // Handle export request
        if ($this->request->is_set_post('export'))
        {
            $format = $this->request->variable('export_format', '');
            $this->export_logs($format);
        }

        // Delete logs
        if ($this->request->is_set_post('delete_all') && confirm_box(true))
        {
            $sql = 'DELETE FROM ' . $this->table_prefix . 'search_logs';
            $this->db->sql_query($sql);

            trigger_error($this->language->lang('ACP_SEARCHLOGGER_LOGS_DELETED') . adm_back_link($this->u_action));
        }
        else if ($this->request->is_set_post('delete_all'))
        {
            confirm_box(false, $this->language->lang('ACP_SEARCHLOGGER_CONFIRM_DELETE'), build_hidden_fields(array(
                'delete_all' => 1,
            )));
        }
		
		
		
		
		
		
		
		
		
		 // Count total logs
        $sql = 'SELECT COUNT(*) as total_logs FROM ' . $this->table_prefix . 'search_logs';
        $result = $this->db->sql_query($sql);
        $total_logs = (int) $this->db->sql_fetchfield('total_logs');
        $this->db->sql_freeresult($result);

        // Get logs
        $sql_array = [
            'SELECT' => 'l.*, u.username, u.user_colour',
            'FROM' => [
                $this->table_prefix . 'search_logs' => 'l',
            ],
            'LEFT_JOIN' => [
                [
                    'FROM' => [USERS_TABLE => 'u'],
                    'ON' => 'l.user_id = u.user_id',
                ],
            ],
            'ORDER_BY' => 'l.search_time DESC',
        ];

        $sql = $this->db->sql_build_query('SELECT', $sql_array);
        $result = $this->db->sql_query_limit($sql, $per_page, $start);

        $logs = [];
        while ($row = $this->db->sql_fetchrow($result))
        {
            $logs[] = [
                'username' => $row['username'],
                'user_colour' => $row['user_colour'],
                'search_query' => $row['search_query'],
                'search_time' => $this->user->format_date($row['search_time']),
            ];
        }
        $this->db->sql_freeresult($result);

        // Generate pagination
        global $phpbb_container;
        $pagination = $phpbb_container->get('pagination');
        $pagination->generate_template_pagination(
            $this->u_action, 
            'pagination', 
            'start', 
            $total_logs, 
            $per_page, 
            $start
        );

        $this->template->assign_vars(array(
            'TOTAL_LOGS'     => $total_logs,
            'U_ACTION'       => $this->u_action,
            'S_LOGS_FOUND'   => !empty($logs),
        ));

        foreach ($logs as $log)
        {
            $this->template->assign_block_vars('log', array(
                'USERNAME'        => $log['username'],
                'USERNAME_COLOUR' => $log['user_colour'],
                'SEARCH_QUERY'    => $log['search_query'],
                'SEARCH_TIME'     => $log['search_time'],
            ));
        }
    }

    /**
     * Export logs to various formats
     *
     * @param string $format Export format (csv, json, txt, excel)
     */
    protected function export_logs($format)
    {
        // Get all logs
        $sql_array = array(
            'SELECT'    => 'l.*, u.username, u.user_colour',
            'FROM'      => array(
                $this->table_prefix . 'search_logs' => 'l',
                USERS_TABLE => 'u',
            ),
            'WHERE'     => 'l.user_id = u.user_id',
            'ORDER_BY'  => 'l.search_time DESC',
        );
        
        $sql = $this->db->sql_build_query('SELECT', $sql_array);
        $result = $this->db->sql_query($sql);
        
        $logs = array();
        while ($row = $this->db->sql_fetchrow($result))
        {
            // Format date based on user language with Italian format
            $current_language = $this->user->lang_name;
            if ($current_language == 'it' || $current_language == 'italian')
            {
                // Italian date format
                $date = new \DateTime();
                $date->setTimestamp($row['search_time']);
                $day_names = array('domenica', 'lunedì', 'martedì', 'mercoledì', 'giovedì', 'venerdì', 'sabato');
                $month_names = array('', 'gennaio', 'febbraio', 'marzo', 'aprile', 'maggio', 'giugno', 'luglio', 'agosto', 'settembre', 'ottobre', 'novembre', 'dicembre');
                
                $day_of_week = $day_names[$date->format('w')];
                $day = $date->format('j');
                $month = $month_names[(int)$date->format('n')];
                $year = $date->format('Y');
                $hour = $date->format('H');
                $minute = $date->format('i');
                $second = $date->format('s');
                
                $search_time_formatted = $day_of_week . ' ' . $day . ' ' . $month . ' ' . $year . ' ore ' . $hour . ':' . $minute . ':' . $second;
            }
            else
            {
                // Default formatting for other languages
                $search_time_formatted = $this->user->format_date($row['search_time']);
            }
            
            $logs[] = array(
                'user_id'        => $row['user_id'],
                'username'       => $row['username'],
                'search_query'   => $row['search_query'],
                'search_time'    => $search_time_formatted,
            );
        }
        $this->db->sql_freeresult($result);
        
        $filename = 'search_logs_' . date('Y-m-d');
        
        switch ($format)
        {
            case 'csv':
                $this->export_csv($logs, $filename);
                break;
                
            case 'json':
                $this->export_json($logs, $filename);
                break;
                
            case 'txt':
                $this->export_txt($logs, $filename);
                break;
                
            case 'excel':
                $this->export_excel($logs, $filename);
                break;
        }
    }
    
    /**
     * Export logs to CSV format
     *
     * @param array $logs Array of log entries
     * @param string $filename Base filename without extension
     */
    protected function export_csv($logs, $filename)
    {
        $csv_data = "User ID,Username,Search Query,Search Results,Search Time\n";
        
        foreach ($logs as $log)
        {
            $csv_data .= $log['user_id'] . ',"' . str_replace('"', '""', $log['username']) . '","' . 
                        str_replace('"', '""', $log['search_query']) . '",' . 
                        $log['search_time'] . "\"\n";
        }
        
        $this->send_file($csv_data, $filename . '.csv', 'text/csv');
    }
    
    /**
     * Export logs to JSON format
     *
     * @param array $logs Array of log entries
     * @param string $filename Base filename without extension
     */
    protected function export_json($logs, $filename)
    {
        $json_data = json_encode($logs, JSON_PRETTY_PRINT);
        $this->send_file($json_data, $filename . '.json', 'application/json');
    }
    
    /**
     * Export logs to TXT format
     *
     * @param array $logs Array of log entries
     * @param string $filename Base filename without extension
     */
    protected function export_txt($logs, $filename)
    {
        $txt_data = "Search Logs Export\n\n";
        
        foreach ($logs as $log)
        {
            $txt_data .= "User ID: " . $log['user_id'] . "\n";
            $txt_data .= "Username: " . $log['username'] . "\n";
            $txt_data .= "Search Query: " . $log['search_query'] . "\n";
            $txt_data .= "Search Time: " . $log['search_time'] . "\n";
            $txt_data .= "----------------------------------------\n";
        }
        
        $this->send_file($txt_data, $filename . '.txt', 'text/plain');
    }
    
    /**
     * Export logs to Excel format
     *
     * @param array $logs Array of log entries
     * @param string $filename Base filename without extension
     */
    protected function export_excel($logs, $filename)
    {
        // Check if PhpSpreadsheet is available
        if (!class_exists('\PhpOffice\PhpSpreadsheet\Spreadsheet'))
        {
            trigger_error($this->language->lang('ACP_SEARCHLOGGER_PHPSPREADSHEET_REQUIRED') . adm_back_link($this->u_action), E_USER_WARNING);
        }
        
        try 
        {
            $spreadsheet = new \PhpOffice\PhpSpreadsheet\Spreadsheet();
            $sheet = $spreadsheet->getActiveSheet();
            
            // Set headers
            $sheet->setCellValue('A1', 'User ID');
            $sheet->setCellValue('B1', 'Username');
            $sheet->setCellValue('C1', 'Search Query');
            $sheet->setCellValue('D1', 'Search Time');
            
            // Add logs data
            $row = 2;
            foreach ($logs as $log)
            {
                $sheet->setCellValue('A' . $row, $log['user_id']);
                $sheet->setCellValue('B' . $row, $log['username']);
                $sheet->setCellValue('C' . $row, $log['search_query']);
                $sheet->setCellValue('D' . $row, $log['search_time']);
                $row++;
            }
            
            // Auto size columns
            foreach (range('A', 'E') as $col) 
            {
                $sheet->getColumnDimension($col)->setAutoSize(true);
            }
            
            // Create Excel file
            $writer = new \PhpOffice\PhpSpreadsheet\Writer\Xlsx($spreadsheet);
            
            // Save to a temporary file
            $temp_file = tempnam(sys_get_temp_dir(), 'phpbb_searchlogs_');
            $writer->save($temp_file);
            
            // Send the file to the browser
            $this->send_file_from_path($temp_file, $filename . '.xlsx', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        }
        catch (\PhpOffice\PhpSpreadsheet\Exception $e) 
        {
            // Specific PhpSpreadsheet exception
            trigger_error($this->language->lang('ACP_SEARCHLOGGER_EXCEL_EXPORT_ERROR', $e->getMessage()) . adm_back_link($this->u_action), E_USER_WARNING);
        }
        catch (\Exception $e) 
        {
            // Generic exception
            trigger_error($this->language->lang('ACP_SEARCHLOGGER_EXCEL_EXPORT_ERROR', $e->getMessage()) . adm_back_link($this->u_action), E_USER_WARNING);
        }
    }
    
    /**
     * Send a file to the browser from a string
     *
     * @param string $data File content
     * @param string $filename Filename to use for download
     * @param string $mimetype MIME type of the file
     */
    protected function send_file($data, $filename, $mimetype)
    {
        header('Content-Type: ' . $mimetype);
        header('Content-Disposition: attachment; filename="' . $filename . '"');
        header('Content-Length: ' . strlen($data));
        header('Pragma: no-cache');
        header('Expires: 0');
        
        echo $data;
        exit;
    }
    
    /**
     * Send a file to the browser from a file path
     *
     * @param string $filepath Path to the file
     * @param string $filename Filename to use for download
     * @param string $mimetype MIME type of the file
     */
    protected function send_file_from_path($filepath, $filename, $mimetype)
    {
        header('Content-Type: ' . $mimetype);
        header('Content-Disposition: attachment; filename="' . $filename . '"');
        header('Content-Length: ' . filesize($filepath));
        header('Pragma: no-cache');
        header('Expires: 0');
        
        readfile($filepath);
        unlink($filepath); // Delete the temporary file
        exit;
    }
}