<?php
/**
 * Search Logger - Migrations
 *
 * @package   SearchLogger
 * @copyright (c) 2025 salvocortesiano
 * @license   GNU General Public License, version 2 (GPL-2.0)
 */

namespace salvocortesiano\searchlogger\migrations;

class salvocortesiano_searchlogger_1_0_0_schema extends \phpbb\db\migration\migration
{
    /**
     * Check if migration is effectively installed
     *
     * @return bool True if migration is installed, False if not
     */
    public function effectively_installed()
    {
        return $this->db_tools->sql_table_exists($this->table_prefix . 'search_logs');
    }

    /**
     * Migration dependencies
     *
     * @return array Array of migration dependencies
     */
    public static function depends_on()
    {
        return array('\phpbb\db\migration\data\v32x\v321');
    }
    /**
     * Returns an array of tables schema
     *
     * @return array Tables schema
     */
    public function update_schema()
    {
        return array(
            'add_tables' => array(
                $this->table_prefix . 'search_logs' => array(
                    'COLUMNS' => array(
                        'log_id'         => array('UINT', null, 'auto_increment'),
                        'user_id'        => array('UINT', 0),
                        'search_query'   => array('TEXT', ''),
                        'search_time'    => array('TIMESTAMP', 0),
                    ),
                    'PRIMARY_KEY' => 'log_id',
                    'KEYS' => array(
                        'user_id'       => array('INDEX', 'user_id'),
                        'search_time'   => array('INDEX', 'search_time'),
                    ),
                ),
            ),
        );
    }

    /**
     * Returns an array of actions to reverse the schema changes
     *
     * @return array Array of table schema changes
     */
    public function revert_schema()
    {
        return array(
            'drop_tables' => array(
                $this->table_prefix . 'search_logs',
            ),
        );
    }

    /**
     * Install the config settings
     *
     * @return array Array of data update instructions
     */
    public function update_data()
    {
        return array(
            // Add config settings
            array('config.add', array('searchlogger_enabled', 0)),
            array('config.add', array('searchlogger_retention_days', 30)),
        );
    }
}
